/*
// musixpss.c - MusiXTeX Extention Package: PostScript Slurs by MetaPost
//              Support Program
//
// This is a part of 
//     `MusiXTeX Extention Package: PostScript Slurs by MetaPost.'
//
// Copyright (C) 2001-2003 Hiroaki MORIMOTO (Kuuku)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// Author:
//   Hiroaki MORIMOTO
//     2-5-18-409, Kita, Kunitachi-city, Tokyo 186-0001 Japan
//     E-mail: CQX05646@nifty.com
//
// based on musixtex.tex T.99, line 508-1233.
// MusiXTeX : (C) Daniel Taupin et al.
//
// January 5, 2003 version 0.50
//
*/

#define PROGDATE "January 5, 2003"
#define PROGVER "0.50"

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#define PATH_DELIMITER '\\'    /* for MS-DOS, MS-Windows */
/* #define PATH_DELIMITER '/'    for UNIX and clone */

int process_makeslurs(char *fname_base);
int readparams(char *buf, char *params[], int maxparams);
int process_clean(char *fname_base);


/*--------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
	char	fname_base[256];
	char	*argv_file[256], *argv_option[256];
	int 	argc_file=0, argc_option=0;
	int 	i, r;
	char	*wp;
	int 	operation;
	
	printf("MusiXTeX Extention: PostScript Slurs by MetaPost\n"
	       "Support Program version "PROGVER" <"PROGDATE">\n");
	if(argc<2){
		printf(
			"\tusage: musixpss [-options] <filename>\n"
			"\tsee documentation for details.\n"
		);
		return(1);
	}
	
	/* sort filenames and options */
	for(i = 0; i < argc; i++){
		if(argv[i][0] == '-'){
			argv_option[argc_option] = argv[i];
			printf("option: %s\n", argv_option[argc_option]);
			argc_option++;
		}else{
			argv_file[argc_file] = argv[i];
			printf("file: %s\n", argv_file[argc_file]);
			argc_file++;
		}
	}
	argv_file[argc_file] = NULL;
	argv_option[argc_option] = NULL;
	
	/* perform filenames */
	strcpy(fname_base, argv_file[1]);
	if((wp=strrchr(fname_base, '.'))!=NULL && wp>strrchr(argv_file[1], PATH_DELIMITER)){
		/* There's filename extention */
		*wp='\0';
	}
	
	/* operation select */
	operation = 0;
	for(i = 0; i < argc_option; i++){
		if(strcmp("-clean", argv_option[i]) == 0){
			operation = 1;
		}
	}
	r=0;
	switch(operation){
	case 0: /* process slurs */
		r = process_makeslurs(fname_base);
		break;
	case 1: /* clean intermediate EPS's */
		r = process_clean(fname_base);
		break;
	default:
		break;
	}
	
	return(r);
} /* end of main() */


/*--------------------------------------------------------------------------*/
int process_makeslurs(char *fname_base)
{
	int 	r;
	FILE	*fsl;      /* .slu file */
	FILE	*fmp;      /* this program's output == MetaPost input */
	char	fname_slu[256], fname_mp[256];
	char	slurlenstr[256];
	char	buf[1024];
	int 	slurid=0;
	double	musicsize;
	double	thickness_adj;
	int 	slope;
	double	slurlen;
	int 	height;
	int 	invert;    /* flag of \invertslur */
	int 	dotted;    /* flag of \dotted */
	char	*params[256];

	strcpy(fname_slu, fname_base);
	strcat(fname_slu, ".slu");
	strcpy(fname_mp, fname_base);
	strcat(fname_mp, ".mp");
	
	
	if((fsl=fopen(fname_slu, "r"))==NULL){
		printf("Error: can't open file `%s'\n", fname_slu);
		return(1);
	}
	
	if((fmp=fopen(fname_mp, "w"))==NULL){
		printf("Error: can't create file `%s'\n", fname_mp);
		return(1);
	}
	
	printf("Now processing %s -> %s\n", fname_slu, fname_mp);
	
	fprintf(fmp, "%% generated by musixpss v"PROGVER" from %s\n", fname_slu);
	fprintf(fmp, "input musixpss.mp;\n");
	
	while(fgets(buf, 1024, fsl)!=NULL){
		printf(".");
		
		r = readparams(buf, params, 7);
		if(r < 7){
			printf("Error: slu file error, not enough parameter.\n");
			return(1);
		}
		
		musicsize	= atof(params[0]);
		thickness_adj = atof(params[1]);
		slope		= atoi(params[2]);
		slurlen		= atof(strcpy(slurlenstr, params[3]) );
		height		= atoi(params[4]);
		invert		= atoi(params[5]);
		dotted		= atoi(params[6]);
		
		slurid++;
		
		fprintf(fmp, "beginfig(%d)\n", slurid);
		fprintf(fmp, "\tux:=%s;\n", slurlenstr);
		fprintf(fmp, "\tuy:=%.5fpt;\n", musicsize/8.0*height);
		fprintf(fmp, "\tsy:=%.5fpt;\n", musicsize/8.0*slope);
		fprintf(fmp, "\tthick:=%.2fpt;\n", musicsize*thickness_adj/20.0);
		fprintf(fmp, "\tdashunit:=%.2fpt;\n", musicsize/4.0);
		fprintf(fmp, "\tdashduty:=0.7;\n");
		fprintf(fmp, "\tinvert:=%d;\n", invert);
		fprintf(fmp, "\tmakedotted:=%d;\n", dotted);
		fprintf(fmp, "\tdrawslur;\n");
		fprintf(fmp, "endfig;\n\n");
	}
	fprintf(fmp, "end.\n");
	
	printf("\nCompleted. %d slur(s) performed.\n", slurid);
	return(0);
}


/*--------------------------------------------------------------------------*/
int readparams(char *buf, char *params[], int maxparams)
{
	int 	r = 0;
	char	*wp;
	
	while( (wp=strtok(r == 0 ? buf : NULL, " ,\n") ) != NULL){
		params[r++] = wp;
		if(r >= maxparams){
			break;
		}
	}
	return(r);
}


/*--------------------------------------------------------------------------*/
int process_clean(char *fname_base)
{
	FILE	*fsl;      /* .slu file */
	char	fname_slu[256], fname_eps[256];
	char	buf[1024];
	int 	slurid=0;

	strcpy(fname_slu, fname_base);
	strcat(fname_slu, ".slu");
	if((fsl=fopen(fname_slu, "r"))==NULL){
		printf("Error: can't open file `%s'\n", fname_slu);
		return(1);
	}
	
	printf("Now removing intermediate EPS files for %s:\n", fname_slu);
	
	while(fgets(buf, 1024, fsl)!=NULL){
		slurid++;
		sprintf(fname_eps, "%s.%d", fname_base, slurid);
		printf(".");
		if(remove(fname_eps) ){
			printf("Error: can't remove %s. Something's wrong. Continue? [y/n] ", fname_eps);
			gets(buf);
			if(buf[0] != 'y' && buf[0] != 'Y'){
				return(1);
			}
		}
	}
	
	printf("\nCompleted. %d intermediate EPS file(s) removed.\n", slurid);
	return(0);
}


